const fs = require('fs')
const path = require('path')
const chalk = require('chalk')
const webpack = require('webpack')
const process = require('process')
const docGen = require('sd-doc-gen')
const vuepress = require('vuepress')

const root = process.cwd()

// 将src下的文件夹的名字都设为alias
function aliasProvider () {
  const aliasObj = {
    'vue$': 'vue/dist/vue.esm.js',
    '@': path.join(root, 'src'),
    'src': path.join(root, 'src')
  }
  fs.readdirSync(path.join(root, 'src'))
    .filter(fileName => fs.lstatSync(path.join(root, 'src', fileName)).isDirectory())
    .forEach(fileName => {
      aliasObj[fileName] = path.join(root, 'src', fileName)
    })
  return aliasObj
}
const alias = aliasProvider()

module.exports = (api, projectOptions) => {
  api.chainWebpack(webpackConfig => {
    // alias
    Object.keys(alias).forEach(key => {
      webpackConfig.resolve.alias.set(key, alias[key])
    })
    // 避免nginx代理之后因为不断check host导致报错
    webpackConfig.devServer.disableHostCheck(true)
    // 移除 prefetch 插件
    webpackConfig.plugins.delete('prefetch')
    // 移除preload
    webpackConfig.plugins.delete('preload')
    // dll
    webpackConfig.plugin('DllReferencePlugin')
      .use(webpack.DllReferencePlugin, [{
        context: root,
        manifest: require('./manifest.json')
      }])
  })

  api.configureWebpack(webpackConfig => {
    // 修改 webpack 配置
    // 或返回通过 webpack-merge 合并的配置对象
  })

  api.registerCommand('doc', args => {
    docGen(root)
    vuepress.dev(path.join(root, 'docs'), {
      open: true,
      theme: '@vuepress/default'
    })
  })
  api.registerCommand('dll', args => {
    webpack(require('./config/webpack.dll.conf'), (err, stats) => {
      if (err) throw err
      if (stats.hasErrors()) {
        console.log(chalk.red('  Build failed with errors.\n'))
        process.exit(1)
      }
      console.log(chalk.cyan('  Build complete.\n'))
    })
  })
}
